// src/components/ItemDetails.js
import React, { useState, useEffect, useRef, useCallback } from 'react';
import { TextField, Button, Container, Avatar, Tooltip } from '@mui/material';
import { PRIMARY_COLOR, SECONDARY_COLOR } from '../App';
import { api } from '../services/api';

export default function ItemDetails({ item, token, onSave, onClose, boxId }) {
  const [name, setName] = useState(item.name);
  const [description, setDescription] = useState(item.description);
  const [imagePath, setImagePath] = useState(item.image_path || '');
  const [imageSrc, setImageSrc] = useState('/images/default.jpg');
  const fileInputRef = useRef(null);
  const [imageOverlayVisible, setImageOverlayVisible] = useState(false);
  const [boxName, setBoxName] = useState('');
  const [boxes, setBoxes] = useState([]);
  const [selectedBoxId, setSelectedBoxId] = useState(item.box_id);

  useEffect(() => {
    const fetchBoxes = async () => {
      try {
        const response = await api.boxes.getAll(token);
        setBoxes(response.data);
      } catch (error) {
        console.error('Error fetching boxes:', error);
      }
    };
    fetchBoxes();
  }, [token]);

  const handleBoxChange = (event) => {
    const newBoxId = event.target.value;
    setSelectedBoxId(newBoxId);
  };

  useEffect(() => {
    const getBoxDetails = async (boxId) => {
      try {
        const boxIdNumber = +boxId;
        if (isNaN(boxIdNumber)) {
          console.error('Invalid boxId:', boxId);
          return;
        }
        const response = await api.boxes.getAll(token);
        const box = response.data.find(b => b.ID === boxIdNumber);
        if (box) {
          setBoxName(box.name);
        }
      } catch (error) {
        console.error('Error fetching box details:', error);
      }
    };

    if (selectedBoxId !== item.box_id) {
      getBoxDetails(selectedBoxId);
    } else if (item.box_id) {
      getBoxDetails(item.box_id);
    }
  }, [selectedBoxId, token, item.box_id]);

  useEffect(() => {
    const fetchItemImage = async () => {
      try {
        const response = await api.items.getImage(token, item.ID);
        const reader = new FileReader();
        reader.onload = () => setImageSrc(reader.result);
        reader.readAsDataURL(response.data);
      } catch (error) {
        setImageSrc('/default.jpg');
      }
    };
    fetchItemImage();
  }, [item.ID, token]);

  const handleImageUpload = useCallback(async () => {
    if (!fileInputRef.current.files[0]) return;

    const formData = new FormData();
    formData.append('image', fileInputRef.current.files[0]);

    try {
      const response = await api.items.uploadImage(token, item.ID, formData);
      return response.data.imagePath;
    } catch (error) {
      console.error('Image upload failed:', error);
    }
  }, [item.ID, token]);

  const handleSave = useCallback(async () => {
    let imagePath;
    if (fileInputRef.current.files[0]) {
      imagePath = await handleImageUpload();
    }

    try {
      await api.items.update(token, item.ID, {
        name,
        description,
        box_id: +selectedBoxId,
      });
      onSave();
    } catch (error) {
      console.error('Item update failed:', error);
    }
  }, [item.ID, name, description, selectedBoxId, token, onSave, handleImageUpload]);

  const handleImageError = (e) => {
    e.target.src = '/images/default.jpg';
  };

  const handleAvatarClick = () => {
    setImageOverlayVisible(true);
  };
  
  const handleCloseOverlay = () => {
    setImageOverlayVisible(false);
  };

  return (
    <Container>
      <h3>Edit Item: {item.name}</h3>

      <Tooltip title="Click to enlarge">
        <Avatar
          src={imageSrc}
          alt={name}
          onError={handleImageError}
          sx={{ width: 200, height: 200, marginBottom: '16px' }}
          onClick={handleAvatarClick}
        />
      </Tooltip>  
      {imageOverlayVisible && (
        <div className="image-overlay">
          <img src={imageSrc} alt={name} />
          <button className="close-button" onClick={handleCloseOverlay}>
            Close
          </button>
        </div>
      )}
      <TextField
        label="Item Name"
        variant="outlined"
        fullWidth
        margin="normal"
        value={name}
        onChange={(e) => setName(e.target.value)}
      />
      <TextField
        label="Item Description"
        variant="outlined"
        fullWidth
        margin="normal"
        value={description}
        onChange={(e) => setDescription(e.target.value)}
      />
      <TextField 
        label="Item Image Path" 
        variant="outlined" 
        fullWidth 
        margin="normal"
        value={imagePath}
        onChange={(e) => setImagePath(e.target.value)} 
        sx={{ display: 'none' }}
      />
      <input 
        type="file" 
        accept="image/*" 
        ref={fileInputRef} 
        style={{ display: 'none' }} 
        id="editItemImageUpload"
      />
      <select value={selectedBoxId} onChange={handleBoxChange}>
        <option value="">No box</option>
        {boxes.map((box) => (
          <option key={box.ID} value={box.ID}>
            {box.name}
          </option>
        ))}
      </select>     
      <br/>
      <br/>
      <Button sx={{ backgroundColor: PRIMARY_COLOR, borderBottom: '1px solid', borderColor: '#444', color: SECONDARY_COLOR }} variant="contained" component="label" htmlFor="editItemImageUpload"> 
        Upload Image
        <input 
          type="file" 
          accept="image/*" 
          id="image-upload" 
          style={{ display: 'none' }} 
          onChange={(e) => {
            setImagePath(e.target.files[0].name); 
          }}
        />
      </Button>

      <Button sx={{ backgroundColor: PRIMARY_COLOR, borderBottom: '1px solid', borderColor: '#444', color: SECONDARY_COLOR }} variant="contained" color="primary" onClick={handleSave}>
        Save Changes
      </Button>
      <Button sx={{ backgroundColor: PRIMARY_COLOR, borderBottom: '1px solid', borderColor: '#444', color: SECONDARY_COLOR }} variant="contained" color="primary" onClick={onClose}>Close</Button>
    </Container>
  );
}